/*-------------------------------------------------------------------------------------

BrowsFolders.C
http://msdn.microsoft.com/en-us/library/windows/desktop/bb762115(v=vs.85).aspx

SHBrowseForFolder function
Displays a dialog box that enables the user to select a Shell folder.
Syntax

C++

PIDLIST_ABSOLUTE SHBrowseForFolder(
  _In_  LPBROWSEINFO lpbi
);

Parameters
lpbi [in]
Type: LPBROWSEINFO
A pointer to a BROWSEINFO structure that contains information used to display the dialog box.
Return value

Type: PIDLIST_ABSOLUTE
Returns a PIDL that specifies the location of the selected folder relative to the root of the namespace. If the user chooses the Cancel button in the dialog box, the return value is NULL.
It is possible that the PIDL returned is that of a folder shortcut rather than a folder. For a full discussion of this case, see the Remarks section.
Remarks

For Windows Vista or later, it is recommended that you use IFileDialog with the FOS_PICKFOLDERS option rather than the SHBrowseForFolder function. This uses the Open Files dialog in pick folders mode and is the preferred implementation.
Needs a Better Example

You must initialize Component Object Model (COM) before you call SHBrowseForFolder. If you initialize COM using CoInitializeEx, you must set the COINIT_APARTMENTTHREADED flag in its dwCoInit parameter. You can also use CoInitialize or OleInitialize, which always use apartment threading. If you require drag-and-drop functionality, OleInitialize is recommended because it initializes the required OLE as well as COM.
Note  If COM is initialized using CoInitializeEx with the COINIT_MULTITHREADED flag, SHBrowseForFolder fails if the calling application uses the BIF_USENEWUI or BIF_NEWDIALOGSTYLE flag in the BROWSEINFO structure.
It is the responsibility of the calling application to call CoTaskMemFree to free the IDList returned by SHBrowseForFolder when it is no longer needed.


 ------------------------- How To Use It -------------------------

#include "FolderBrowserDialog.h"
...
FolderBrowserDialog *folderBrowserDialog1 = new FolderBrowserDialog();

if (folderBrowserDialog1->ShowDialog())
{
    MessageBox(0, folderBrowserDialog1->SelectedPath, _T("Selected path:"),
               MB_OK | MB_ICONINFORMATION);
}
...
folderBrowserDialog1->Flags &= ~BIF_USENEWUI;
folderBrowserDialog1->Flags |= BIF_RETURNONLYFSDIRS;
folderBrowserDialog1->Title = _T("Choose a folder please. Or try to create new ;)");

if (folderBrowserDialog1->ShowDialog())
{
    MessageBox(0, folderBrowserDialog1->SelectedPath, _T("Selected path:"),
               MB_OK | MB_ICONINFORMATION);
}

*-----------------------------------------------------------------------------------------*/

//#pragma once

//#include "main.h"
#include <Windows.h>
#include <Shlobj.h>

  #define dputs(a)  ((void)0)

//#pragma comment(lib, "Shell32.lib");

#if 0
class FolderBrowserDialog
{
public:
    FolderBrowserDialog(void);
    
    int Flags;
    HWND Owner;
    TCHAR *SelectedPath;
    TCHAR *Title;

    bool ShowDialog();
};
#include "FolderBrowserDialog.h"

FolderBrowserDialog::FolderBrowserDialog(void)
{
    this->Flags = BIF_USENEWUI;
    this->Owner = 0;
    this->Title = 0;
}

#endif


/*---------------------------------------------------------------------

  GetPathFromUserDlg()

  Open a Browse Folder Dialgue so user can pick a path


 ---------------------------------------------------------------------*/

BOOL GetPathFromUserDlg(HWND Owner, char *pSelectedPath)
{
char *Title = "Please choose a folder, please. :-)";
BROWSEINFO bi;
LPITEMIDLIST pIDL;


    memset(&bi, 0, sizeof(bi));
	*pSelectedPath = 0;

    bi.hwndOwner = Owner;
    bi.lpszTitle = Title;
    bi.ulFlags   = BIF_RETURNONLYFSDIRS|BIF_BROWSEINCLUDEFILES|BIF_EDITBOX;

    OleInitialize(NULL);

    pIDL = SHBrowseForFolder(&bi);
    if (pIDL == NULL)
    {
		dputs("GetPathFromUserDlg(): SHBrowseForFolder() failed");
        return FALSE;
    }
    
    if(!SHGetPathFromIDList(pIDL, pSelectedPath))
    {
		dputs("GetPathFromUserDlg(): SHGetPathFromIDList failed");
        CoTaskMemFree(pIDL);
        return FALSE;
    }

    CoTaskMemFree(pIDL);
    OleUninitialize();

	dputs("GetPathFromUserDlg(): ...everything worked, returning TRUE");

    return TRUE;
}




/*---------------------------------------------------------------------
 * GetFileName - get a file name using common dialog stuff

SAMPLE USAGE:

HANDLE save_instance=0;

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine,
    int nCmdShow)
	save_instance=hInstance;
..
...


BOOL FileOpen( HWND   hWnd, STOCK *s, char *szpfilename)
{
char szfname[_MAX_PATH],  sz[50], *ptr;
int fh;

	if(!szpfilename)
	{
		if(!GetFileName( hWnd, szfname ) )  //not dragndrop...go get filename from user
			return FALSE;
	}
	else
		strcpy(szfname,szpfilename);


 ---------------------------------------------------------------------*/
BOOL GetFileName( HWND hWnd, char *pStrFileName )
{
//char  filterList[] = { "CSV (*.csv)\0*.csv\0\0" };
char  filterList[] = { "Executable (*.exe, *.com, *.bat)\0*.exe;*.com;*.bat\0\0" };
//char  filterList[] = { "*.COM;*.EXE;*.BAT"};
OPENFILENAME   of;
int  rc=0;
extern HANDLE save_instance;


    pStrFileName[ 0 ] = '\0';
    memset( &of, 0, sizeof( OPENFILENAME ) );
    of.lStructSize = sizeof( OPENFILENAME );
    of.hwndOwner = hWnd;
	of.hInstance = save_instance;      /*See USAGE in header */
    of.lpstrFilter = (LPSTR) filterList;
    of.lpstrDefExt = "";
    of.nFilterIndex = 1L;
    of.lpstrFile = pStrFileName;
    of.nMaxFile = MAX_PATH;
    of.lpstrTitle = "Get Executable";
    of.Flags = OFN_HIDEREADONLY;
    rc = GetOpenFileName( &of );
    return( rc );

} /* GetFileName */

