///////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////// DEBUGGING FUNCTION  ////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

#include "SRCHFILE.h"

int logging_on=1;

#if DEBUG_PUTS
//#error what

static FILE *stream = NULL;
char *dbugFname = "logging.log";

/*----------------------------------------------------------------------
	
	dputs()
	
	debug logging 
    
  2014.02.13  
    Moved dputs() to SA_LIB.C to access the gPAW struct, et al.
	Enhances trouble-shooting

  2014.6.13 - opening a 2nd console window for real time output

*----------------------------------------------------------------------*/
void dputs(char *msg, ... )
{
va_list  argptr;          /* Argument list pointer    */
char szBuff[1700];         /* Buffer to build string into   */
static int found_explosion_rate_culprit = 0;

	if(!logging_on)
		return;

    va_start( argptr, msg );      /* Initialize va_ functions */
    vsprintf( szBuff, msg, argptr );   /* prints string to buffer  */
    va_end( argptr );         /* Close va_ functions      */

	if(!stream)
	{
	struct tm *newtime;
	time_t l_time;
		
		time( &l_time ); 
		newtime = localtime( &l_time );
		
		/*
		"w+"  Opens an empty file for both reading and writing. If the given file exists, its contents are destroyed.
		
		"a+"  Opens for reading and appending; the appending operation includes the removal of the EOF marker 
				before new data is written to the file and the EOF marker is restored after writing is complete; 
				creates the file first if it doesnt exist.
		*/
		if( (stream = fopen( dbugFname, "w+" )) == NULL ) /* _O_RDWR | _O_APPEND | _O_CREAT + BINARY */
		{
			sprintf(szBuff, "ERROR:  The debug logging file '%s' could not not be opened\n", dbugFname);
			MessageBox(NULL, szBuff, "ERROR", MB_ICONEXCLAMATION );
			exit(0);
		}
		fputs("\n\n--------------------------------------------------------------------------\n", stream);
		fprintf(stream, "INFO: Session Logging beginning %s \n", asctime( newtime ));
		fputs("--------------------------------------------------------------------------\n", stream);
	}
	if(LAST_CHAR(szBuff) != '\n')
		strcat(szBuff, "\n");

	//strcat(szBuff, str_dispbuff_status());

	if(stream)
	{
		/* SEEK_END -- because log reader may also have this file open, and moved the file pointer.
		** Hasn't caused a misfire yet -- but -- safe side*/
		fseek(stream, 0, SEEK_END); 
		fputs(szBuff, stream);
		fflush(stream);
	}
}
#endif //DEBUG_PUTS







/*-----------------------------------------------------------------

  oops()

  Error message handler -- in use
	 
   SAMPLE code from a website


*-------------------------------------------------------------*/
static void oops(const char *message)
{
char *buf;
DWORD error;
 
	buf = NULL;
	error = GetLastError();
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
	    FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
	    NULL, error, 0, buf, 0, NULL);
 
	if(buf) 
	{
		dputs("ERROR:  %ls: %ls", message, buf);
		LocalFree(buf);
	} 
	else 
	{
		/* FormatMessageA() failed. */
		dputs("ERROR:  %ls: unknown error 0x%x\n",  message, error);
	}
}



/*-----------------------------------------------------------------

  get_set_ftime()  Optionaly gets/sets a file date-time stamp
  Calls GetFileTime() / SetFileTime()
  Adapted from _set_fwtime() Module SC_FTIME.C of F:\PROJECTS\32bit\SC


NOTES:------------
 SetFileTime()

The SetFileTime function sets the date and time that a file was created, last accessed, or last modified. 

BOOL SetFileTime(
  HANDLE hFile,                      // handle to the file
  CONST FILETIME *lpCreationTime,    // time the file was created
  CONST FILETIME *lpLastAccessTime,  // time the file was last 
  CONST FILETIME *lpLastWriteTime    // time the file was last written
);
 
Parameters
 
  Handle to the file for which to set the dates and times. 
    The file handle must have been created with GENERIC_WRITE access to the file. 

  lpCreationTime 
	Pointer to a FILETIME structure that contains the date and time the file was created. 
	This parameter can be NULL if the application does not need to set this information. 

  lpLastAccessTime 
	Pointer to a FILETIME structure that contains the date and time the file was last accessed. 
	The last access time includes the last time the file was written to, read from, or (in the case of executable files) run. This parameter can be NULL if the application does not need to set this information. 
  
  lpLastWriteTime  *********** this is the only one we're interested in ********
	Pointer to a FILETIME structure that contains the date and time the file was last written to. 
	This parameter can be NULL if the application does not want to set this information. 

  Return Values
	If the function succeeds, the return value is nonzero.
	If the function fails, the return value is zero. To get extended error information, call GetLastError. 

	http://msdn.microsoft.com/en-us/library/windows/desktop/aa363858(v=vs.85).aspx
	See examples there
	http://msdn.microsoft.com/en-us/library/windows/desktop/bb540534(v=vs.85).aspx

*------------------------------------------------------------------*/
BOOL get_set_ftime (char *pathname, FILETIME *fileTime, BOOL set)
{
HANDLE fh;
BOOL res = TRUE;
char *psAction="set";

	/* Open the file using CreateFile() so we can  1. Get the  file time stamp, and 2. Update time stamp */
	fh = CreateFile(pathname, GENERIC_READ | FILE_WRITE_ATTRIBUTES,  0, NULL, OPEN_EXISTING, 0, NULL);
	if(fh == INVALID_HANDLE_VALUE) 
	{
		oops(pathname);
		return 0;
	}

	if(set)
	{
		psAction="setting";
		if(!SetFileTime(fh, NULL, NULL, fileTime))
		{
			oops(pathname);
			res = FALSE;
		}
	}
	else
	{
		psAction="getting";
		if(!GetFileTime(fh, NULL, NULL, fileTime))
		{
			oops(pathname);
			res = FALSE;
		}
	}


	if(res)
	{
		dputs("get_set_ftime() success %s timestamp file: %s.", psAction, pathname);
	}
	else
	{
		global_error_flag = TRUE;
		dputs("ERROR: get_set_ftime() Failed %s timestamp file: %s ", psAction, pathname);
	}
 
	CloseHandle(fh);
    return res;
}


