////////////////////////////////////////////////////////////////////////////////////
// StrSrchReplace.c : 
// C program to find and replace all occurrences of a word in file.
//
// Adapted from
// https://codeforwin.org/2018/02/c-program-find-and-replace-a-word-in-file.html
//
////////////////////////////////////////////////////////////////////////////////////

/*
REM xcopy  C:\Users\gregory.spears\Desktop  D:\myWorkStuff\DeskTop\   /C/E/V/H/R/Y/D
xcopy D:\SBC   C:\Users\gs829d\AppData\Roaming\SBC\   /C/E/V/H/R/Y/D

14584

20150828.html
20180622.html

C:/Users/gregory.spears/AppData/Roaming/SBC/Q%20Team%20Link%20Messenger/Archive/Meetings/q_rooms_mp125q1429722391317/Images/ss199b_1429298823101.png
C:/Users/gregory.spears/AppData/Roaming/SBC/Q Team Link Messenger/media/smilies.jar!/happy.gif
C:/Users/gregory.spears/AppData/Roaming/SBC/Q%20Team%20Link%20Messenger/Archive/Persons/br1658/Images/gs829d_1529674549871_small.png
C:\Users\gregory.spears\AppData\Roaming\SBC\Q Team Link Messenger\Archive\Persons\pr260e\Images\gs829d_1478134915070.png
C:\Users\gregory.spears\AppData\Roaming\SBC\Q Team Link Messenger\Archive\Persons\pr260e\Images\gs829d_1477636191933.png

For *.html update "/gregory.spears/" to "/gs829d/"
For *.html update "\gregory.spears\" to "\gs829d\"
*/


#include "SRCHFILE.h"

#define BUFFER_SIZE 20000
#pragma warning( disable : 4996) 


/**
 * Replace all occurrences of a given a word in string.
 */
static int replaceAll(char *str, const char *search_str, const char *replace_str)
{
char *pos, temp[BUFFER_SIZE], original_str[BUFFER_SIZE];
int index = 0;
int owlen;

    owlen = (int)strlen(search_str);

    /*
     * Repeat till all occurrences are replaced. 
     */
    while ((pos = strstr(str, search_str)) != NULL)
    {
        // Backup current line
        strcpy(temp, str);
        strcpy(original_str, str);

        // Index of current found word
        index = (int)(pos - str);

        // Terminate str after word found index
        str[index] = '\0';

        // Concatenate str with new word 
        strcat(str, replace_str);
        
        // Concatenate str with remaining words after 
        // search_str found index.
        strcat(str, temp + index + owlen);
    }
	//Returns non-zero if not match (Hence, TRUE if string is modified)
	return !!strcmp(str,original_str);
}


/*---------------------------------------------------------------------------
  update_file()

  Main entry for the module.   Calls external funtion to replace trings.
  Does the grunt work of parsing file, writing new temp file,
  and finally replacing orig file with temp file.
*---------------------------------------------------------------------------*/
int update_file(char *src_file_with_path, char *search_str, char *replace_str)
{
/* File pointer to hold reference of input file */
FILE * fPtr;
FILE * fTemp;
char path_only[MAX_PATH];
char temp_file_with_path[MAX_PATH];
char buffer[BUFFER_SIZE];
BOOL strings_updated = FALSE;	
FILETIME fileTime;

	/*Exit if bad arg count */
	if(!src_file_with_path)
	{
		global_error_flag = TRUE;
		dputs("ERROR: update_file() was given NULL for filename!  FAIL!\n");
		return FALSE ;
	}
	strcpy(path_only, src_file_with_path);
	StripName(path_only);

	sprintf(buffer, "%d.tmp", time(NULL));
	dir_strcat(temp_file_with_path, path_only, buffer);
	
	dputs("INFO: About to Update File %s \nSearch for:  %s  \nReplace with: %s \nPure Path:  %s\nTemp File: %s\n", 
		src_file_with_path, search_str, replace_str, path_only, temp_file_with_path);


    /*  Open all required files */
    fPtr  = fopen(src_file_with_path, "r");
    fTemp = fopen(temp_file_with_path, "w"); 

    /* fopen() return NULL if unable to open file in given mode. */
    if (fPtr == NULL )
    {
		global_error_flag = TRUE;
        /* Unable to open file hence exit */
		dputs("ERROR: Unable to open src file: %s\n", src_file_with_path);
        return FALSE;
    }

    /* fopen() return NULL if unable to open file in given mode. */
    if ( fTemp == NULL)
    {
		global_error_flag = TRUE;
        /* Unable to open file hence exit */
		dputs("ERROR: Unable to open temp file: %s\n", temp_file_with_path);
        return FALSE;
    }


    /*
     * Read line from source file and write to destination 
     * file after replacing given word.
     */
    while ((fgets(buffer, BUFFER_SIZE, fPtr)) != NULL)
    {
        // Replace all occurrence of word from current line
        if(replaceAll(buffer, search_str, replace_str))
			strings_updated = TRUE;

        // After replacing write it to temp file.
        fprintf(fTemp, "%s", buffer);
	    fflush(fTemp);
	}


    /* Close all files to release resource */
	fflush(fTemp);

	//Sleep(100);
	fclose(fTemp);
    fclose(fPtr);
	//Sleep(50);

	if(strings_updated)
	{
		/* Get Time Stamp orig file */
		get_set_ftime (src_file_with_path, &fileTime, FALSE);

		/* Delete original source file */
		remove(src_file_with_path);
		
		/* Rename temp file as original file */
		rename(temp_file_with_path, src_file_with_path);

		/* Make time stamp match orig file */
		get_set_ftime (src_file_with_path, &fileTime, TRUE);
	}
	else
	{
		remove(temp_file_with_path); 	/* Just Delete temp file */
	}
	dputs("INFO:  Successfully replaced all occurrences of '%s' with '%s' for file %s.", search_str, replace_str, src_file_with_path);

    return TRUE;
}


